\name{dataobj}
\alias{dataobj}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Data object for demos}
\description{
  Builds the data object to run all the demos in yourcast, which is a list
  of cross-sectional identifiers (i.e. csid). The elements of the list are matrices and their
  columns are the dependent variables and covariates for the simulation.  
  }
  \usage{
   dataobj(datapath =NULL, 
           disease = c("allc", "allo", "brst", "cerv",
                       "cvds", "dgst", "homi", "livc", "lung", "malr",
                       "matc", "molp", "omal", "otin", "pern", "rspc",
                       "rspi", "stom", "suic", "trns", "tubr","unin","ward"),
           pop= "population", cov.REDUCE= NULL, cov.FULL= c("tobacco"),
           lagyears = 0, timeseries=TRUE, cntry.vec = NULL, nobvs=NULL,
           covselect=seq(0,10, 5), icode="ggggaatttt",proxfile="adjacency.txt",
           Gnames="cntry.codes.txt", selectages=NULL, verbose=T)
%- maybe also 'usage' for other objects documented here.
}
\arguments{
  \item{datapath}{String. Name of the directory that may have the data
    files. Default:NULL}
  
  \item{disease}{Vector with dependent variable or covariate
    data. You may provide an object with either dependent
    variables or covariates data such as \eqn{c(list(allc), list(gdp))},
    where ''gdp'' may be a covariate. To apply a lag of say, 20 years, to
    ''gdp'' you may  use the formula in yourcast,e.g. \eqn{allc ~ lag(gdp, -20)}.   
    Default:  c("allc", "allo", "brst", "cerv",
    "cvds", "dgst", "homi", "livc", "lung", "malr",
    "matc", "molp", "omal", "otin", "pern", "rspc",
    "rspi", "stom", "suic", "trns",
    "tubr","unin","ward").}
				 
  \item{pop}{String or Object containing the population for all csid and possibly two
    different genders.Default:"population".}
  
  \item{cov.REDUCE}{ Vector or list of data.frames. Covariate data 
    for age and gender independents covariates. The software expands
    the data in this variable to its full length that includes all csid
    for all possible age groups and countries, and that matches the
    length of data in variables disease and pop. Default:NULL}
  
  \item{cov.FULL}{Vector for covariates that may include tobacco, and other
    covariates that are either gender and age dependent, and also includes
    all possible csid. Example, \eqn{c(list(tobacco), list(gdp))}.
    It is not recommended to use this parameter except for the Murray
    database examples presented with the demo files in the package.
    Default:c("tobacco").}

  \item{lagyears}{Scalar.Optional parameter to advance covariates a number
    of years.Use if supplying the covariates separately from dependent
    variables in cov.FULL and/or cov.REDUCE. Otherwise, to lag the
    covariates use the formula in yourcast. Default:0}
  
  \item{timeseries}{Boolean. Append the observation years to the
    matrices for every csid.Default:TRUE }
  
  \item{cntry.vec}{Vector to select specific countries from the dataset,
    if NULL all are selected.Default:NULL }
  
  \item{nobvs}{Scalar. Select only those countries that their numbers of 
    death observations, which are not NA, are larger or
    equal to nobvs, if NULL all are selected. Default:NULL}
  
  \item{covselect}{Vector with age groups to exclude from covariate
    tobacco. Default:\eqn{covselect=seq(0,10, 5)}.}
  
  \item{icode}{ A string indicating how the index variable is coded in the input
    data.See \code{yourcast} for more details. Default:
    icode="ggggaatttt".}
  
  \item{proxfile}{String with the name of the file containing the
    countries proximity values, either 0 (none), 1 (some) and 2
    (very similar countries). Default:proxfile="adjacency.txt".}
  
  \item{Gnames}{The name of the file with the corresponding country codes
    and names.Default:Gnames="cntry.codes.txt".}
  
  
  \item{selectages}{Numeric vector with ages groups to be selected 
    or NULL for all ages in the matrices of csid. Default:NULL.}
  
  \item{verbose}{Boolean. Whether verbose output should be displayed.Defauls:TRUE}
 }
 \details{
   The data element of dataobj list can build from objects in the working
   environment. You may also provide the directory path for the files. The
   disease argument is for files or objects containing data for depvar
   but may also hold covariates that are age-dependent or that have all
   ages explicitly included in the data file or object. The argument ''pop'' is
   reserved for the population and is fully age and gender dependent.The
   argument cov.REDUCE is for those covariates that are not age
   dependent or for covariates that  have only one entry for year and
   country code independently of age group. The software will expand
   those covariates in cov.REDUCE to include entries for every age
   group,country and year.  The argument cov.FULL is only used for
   covariates in the Murray database and, more specifically, for
   ''tobacco''. We recommended not to use it except for the examples in
   the demo of the package, we also recommend
   to use the argument ''disease'' with any covariates that are
   age-dependent, which data has the same length as any of the files or
   object of type depvar.  The argument lagyears is used when the
   only entries in disease are for depvar-type files, since it will
   apply the lag only to those objects that are in cov.REDUCE and/or 
   cov.FULL. Optionally, to apply a lag to any covariates you may use the formula
   argument in the ''yourcast'' function. Applying a lag to any of the
   covariates in rigth-hand-side of the formula will lag all of them for
   the same specified number of years. The other arguments are explained in the description of
   this document. Example
   
   \eqn{datobj <- dataobj(disease=c(list(allc), list(gdp)), pop=list(population),
                  cov.REDUCE=NULL, cov.FULL=NULL,lagyears=0,  timeseries=TRUE, 
                  cntry.vec = NULL, nobvs=NULL, covselect=seq(0,10, 5), 
                  icode="ggggaatttt",proxfile="adjacency.txt",
                  Gnames="cntry.codes.txt", selectages=NULL, verbose=T)}
			 
   The resulting dataobj$data list has elements that are
   matrices and the names of those elements are the csid or the
   combinations of country codes and age groups. Thus we have as many elements as
   countries and age groups. The matrices for each country-age
   group combination contain the data from the depvar, population and covariates.
   The column names have the names of those variables
   and we may append the numbers 2 or 3 depending on whether it is male
   or female to the names of the columns if they are gender dependent.
   The rows are the years of observations. Each matrix element  must expand as many
   rows as the sample.frame specification in the ''yourcast''
   function, starting with the first year that does not contain NA's for
   some of the column.  Say if the \eqn{sample.frame=c(1950,2000,2001, 2030)}, then
   the matrices in dataobj$data have rows for every of those years or
   they may start at some yesr, say 1980, which has some no NA's entry
   in data the matrix. The predicted years from 2001 ahead can contain
   just 'NA' for any of the predicted depvar values. 
   
  
}
\value{
  A list.
  \item{dataobj}{A list, see \code{yourcast} for details.
    The list contains several components. The \code{data} component is a
    list, which every element is a matrix for each cross-sectional unit 
    The csid identifier is a combination of country code and age group.  For
    example USA, country code 2450 and for age-group 45, is csid=245045. The
    columns of the matrices are time-series data for the dependent
    variables (i.e. disease, pop) and the covariates that will be used in 
    the simulation. Other components are \code{index.code} or
    \code{icode}, and \code{G.names} or \code{Gnames}.}
  
  \item{proximity}{ An object with three columns, the first and second
    are country codes and the third is the proximity value,i.e. 0, 1,
    2. See \code{proxfile}.}
  
  ...
}
\references{ \url{http://gking.harvard.edu/yourcast}  }
\author{ Elena Villalon<\email{evillalon@iq.harvard.edu}>}
\note{ ~~further notes~~ 
 See demos; for example: demo(chp.11.4)
 ~Make other sections like Warning with \section{Warning }{....} ~
}
\seealso{ ~~objects to See Also as \code{\link{help}}, ~~~ }
\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.

## The function is currently defined as
function(###datapath= "/nfs/where/export/data/death/WHOdata/Jun2002_data/",
                       datapath =NULL, 
                       disease = c("aids","allc", "allo", "brst", "cerv",
                         "cvds", "dgst", "homi", "livc", "lung", "malr",
                         "matc", "molp", "omal", "otin", "pern", "rspc",
                         "rspi", "stom", "suic", "trns", "tubr","unin","ward"),
                       pop= "population", cov.REDUCE= NULL, cov.FULL= c("tobacco"),
                       lagyears = 30, timeseries=TRUE, 
                       cntry.vec = NULL, nobvs=NULL, covselect=seq(0,10, 5), 
                       icode="ggggaatttt",proxfile="adjacency.txt",
                       Gnames="cntry.codes.txt", selectages=NULL, verbose=T)
{

  dataobj <- list()  

  if(length(disease) <= 0 && length(pop) <= 0 &&
     length(cov.FULL)<= 0 && length(cov.REDUCE) <= 0)
    dataw <- datawho()
  else{
  
    dataw <- datawho(datapath, disease, pop,cov.REDUCE, cov.FULL,lagyears, timeseries,
                     cntry.vec, nobvs, covselect.WHO,selectages, icode,verbose)
  }
 
  dpath <- datapath
  dataobj <- c(dataobj, data=list(dataw), index.code=icode)
  
  if(length(Gnames) <= 0){
    gnames <- NULL 
  }else if(is.character(Gnames)){
    Gnames <- paste(dpath,"/", Gnames, sep="") 
    gnames <- build.Gnames(pfile=Gnames)
 
  }else{    
    gnames <- build.Gnames.obj(pfile=as.matrix(Gnames))
  }

  if(length(gnames) > 0)
    dataobj <- c(dataobj, G.names=list(gnames))
  
  if(length(proxfile) > 0)
    {
      proxobj <- proximity.file(datapath=dpath, filenm = proxfile,
                                filename = proxfile, dir="./tmp", savemat=F)
  
      dataobj <- c(dataobj, proximity=list(proxobj))
    }
  return(dataobj)
}
}
\keyword{file}% at least one, from doc/KEYWORDS

