\name{yourcast}
\alias{yourcast}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Time-series cross-sectional Forecasting}
\description{
  Runs a set of regression models to forecast time-series
  cross-sectional data by either
  considering independent regression in each
  cross-sectional unit or by using a variety of techniques to smooth
  across units.}
\usage{
yourcast(formula=NULL, dataobj=NULL,sample.frame=c(time=1950, 2000, 2001, 2030), 
         standardize=TRUE, elim.collinear=FALSE, tol=0.9999, solve.tol = 1.e-10,svdtol=10^(-10),
         userfile=NULL, savetmp = T, model.frame=F,debug = F,  rerun= "yourcast.savetmp", 
         model="OLS",zero.mean=FALSE, Ha.sigma = 0.3, Ha.sigma.sd= 0.1, Ha.deriv=c(0,0,1),
         Ha.age.weight=0, Ha.time.weight=0, Ht.sigma= 0.3, Ht.sigma.sd=0.1,  Ht.deriv=c(0,0,1),
         Ht.age.weight=0, Ht.time.weight=0,Hat.sigma=0.2,
         Hat.sigma.sd=0.1,Hat.a.deriv=c(0,1),Hat.t.deriv=c(0,1), 
         Hat.age.weight=0,Hat.time.weight=0,Hct.sigma=0.3, Hct.sigma.sd =0.1,
         Hct.t.deriv=1, Hct.time.weight = 0, LI.sigma.mean=0.2,LI.sigma.sd = 0.1, nsample= 500, low.pow=T, verbose=TRUE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{formula}{ A standard \R formula of the form \eqn{y  \sim x1 +
	 x2}, except that an explanatory variable is included for a
    particular cross-section only if it is both listed in the formula
    and available in that cross-section's data set (see
    dataobj). Explanatory variables in the formula but not available for
    a cross-section (or in a cross-sectional dataset but not in the
    formula) are excluded. (For mortality forecasting, the specification
    looks like log(deaths/population)\eqn{ \sim x1 + x2}, with deaths
    and population stored as separate variables in each dataframe.) (May
    be set to NULL if savetmp was set to TRUE on the last run, in which
    case the value of formula will come from the saved file.)}
  
  \item{dataobj}{Four types of inputs are allowed. If (1) dataobj is an
    object (in working memory) or (2) a string with the name of a file on disk, the object must
    contain a list with the following items (the first two of which are required):

    index.code
    
    A string indicating how the index variable is coded in the input
    data. Between 0 and 4 of each of the following characters are used
    in order: - to ignore a character, G for the geographic index (such
    as country), A for a grouped continuous variable like an age group,
    and T for a time period. For example, -GGGGAATTTT means parse
    920004172005 by ignoring 92, using 0004 as the country code, 17 as
    the age group, and 2005 as the year. Default: -GGGGAATTTT.
    
    data
    
    A list of dataframes, one for each cross-sectional unit, with names
    corresponding to the geographic areas and age group cross-sectional
    indices GGGGAA, and rows labeled according to time periods TTTT
    (using notation in index.code, so one item in the list might be
    labeled 432101 for geographic area 4321 and age group 01). Columns
    must include at least one variable common to all dataframes, to be
    used as a dependent variable, and a possibly different set of
    explanatory variables in each cross-sectional unit.
    Every matrix in the data list must contain entries for all of the
    years as specified in the parameter sample.frame, starting with the first year that has some
    variable (column) with an observed value that is not NA, and ending
    with the last predicted year or last element in the vector sample.frame.
    
    G.names, A.names, T.names
    
    Optional two-column dataframes that list all valid numerical codes
    (in the first column, labeled code) and corresponding alphanumeric
    names (optionally in the second column, labeled name) for the
    indices corresponding to the geographic areas in G.names, age groups
    A.names, and time periods T.names. The alphanumeric names are most
    commonly only used for geographic areas, since numerical values for
    age groups and time periods are usually meaingful on their own.
    
    adjacency
    
    Includes codes to construct the symmetric matrix (geographic region
    by geographic region) of proximity scores for geographic smoothing
    used by methods MAP and Bayes. The larger each element of the
    matrix, the more proximate that pair of countries is in the prior; a
    zero element means the two geographic areas are unrelated (the
    diagonal is ignored). This symmetric matrix is constructed by
    yourcast() from the proximity object in dataobj. Each row of
    proximity has three entries, consisting of geographic codes for two
    countries (corresponding to the row and column, and column and row,
    of the symmetric matrix to be built) and a score indicating how
    proximate or similar are the two geographic regions; thus, each row
    represents one element of the symmetric matrix. For convenience,
    geographic regions that are unrelated (and have zero entries in the
    symmetric matrix) may be omitted from proximity. In addition,
    proximity may include rows corresponding to geographic regions not
    in the present analysis.
    

    (3) If dataobj is a string referring to a directory on disk, then
    each element of the list above should be stored in a file in that
    directory, with element data consisting of a subdirectory,
    containing separate ASCII data files. (If this option is chosen, a
    complete data object, called dataobj.Rdata, will be stored in the
    directory named, and it will be loaded automatically if YourCast is
    run again with this chosen option.) (4) The last option is for
    dataobj to be set to NULL, which an be useful if savetmp was set to
    TRUE on the last run, in which case the value of dataobj will come
    from the saved file.}
  
  \item{sample.frame}{A 4 element vector containing, in order, the start
    and end time periods to be used for the observed data and the start
    and end time periods to be forecast. Years identified here that are
    not available for a cross-section are ignored. Default:
    c(1950,2000,2001,2020). (Note that this makes it easy to reserve a
    range of values of the dependent variable for out-of-sample
    forecasting evaluation; our summary() and plot() functions will make
    these comparisons automatically if the out-of-sample data are
    included.) }
  
  \item{standardize}{A boolean; should the covariates in each
    cross-sectional unit be standardized (to zero mean and standard
    deviation of  1)? Standardization is performed for both the in-
    and out-of-sample periods. Default: TRUE.}
  
  \item{elim.collinear}{Boolean. Whether collinearity among covariates
    should be tested and those that are collinear shoul be eliminated.  Default FALSE.}
  
  \item{tol}{Double scalar. Tolerance to find collinearities among
    covariates. Default 0.9999.}
  
  \item{solve.tol}{A real number smaller than $ 1$ that is used in the
    argument of the R-function ``solve'' to invert matrices (see
    description for tol). Default is \eqn{ 1.e-10}.}
  
  \item{svdtol}{A scalar; the tolerance used in inverting a matrix by SVD. Default is \eqn{10^{-10}}.}
  
  \item{userfile}{A string with the name of a file that contains your values for some or all of yourcast()'s arguments. This file contains R code that changes default values of arguments. Default: NULL, E.g., the file might contain:
\deqn{
    index.code <- 30
    data <- "WHOmortalityData"
  }
  If an option is specified in userfile, it takes precidence over
  command line options, so its normally best to specify each option in
  either the userfile or the command line but not both.}

\item{savetmp}{If TRUE, yourcast saves a file in the default directory
  (called yourcast.savetmp) with preliminary calculations. If the value
  of formula or dataobj are missing when yourcast is called, yourcast
  will get their values from this file, if it exists. This saves a
  minute or so of compute time for large data sets and is useful for
  multiple runs on the same data with different formulas specified or
  different prior values. If FALSE, no file is saved. (The structure of
  yourcast.savetmp is for the convenience of yourcast and is not
  intended to be read by the user or saved for more than one run.)
  Default: TRUE.}

  \item{model.frame}{If TRUE, include entire input dataobj in the output
    object. Default: FALSE.}
  
  \item{debug}{Boolean.  It puts the
    environment that contains parameters and arguments of the
    simulation in the user workspace. Default FALSE.}
  
  \item{rerun}{String. The name of the file that is saved in the default
    directory with preliminary calculations; see
    \code{savetmp}. Default:"yourcast.savetmp"} 
  
  \item{model}{A string indicating the forecasting method, including: Bayes maximum a posteriori (``MAP''), Bayes with Gibbs sampling (``Bayes''), Ordinary Least Squares (``OLS''), Poisson (``POISSON''), and Lee-Carter (``LC''). Default: ``OLS''. (We usually recommend MAP.)

    YourCast also includes a procedure to help users set the sigma
    parameters below automatically for the case of model=MAP, and
    smoothing over age, time, or age and time, but for only one
    country. You may do this by running a preprocessing instance of
    YourCast first by setting this parameter to ``EBAYES'' and using
    either the data to be analyzed or a larger data set which is likely
    to have similar or related parameter values. When EBAYES is chosen,
    the YourCast output object will contain only the parameter values to
    feed into the next run of YourCast.}
  
  \item{zero.mean}{A boolean or named vector with a value of \eqn{ \bar\mu}
    for each age group. If TRUE, the prior has zero mean. If FALSE, the
    prior has nonzero mean centered around the observed mean age profile
    (i.e., the average of \eqn{Y} over time and levels of the geographic
    index for each age group). Default: FALSE.}
  
  \item{Ha.sigma}{This can be set in one of three ways: (1) a scalar
    which sets \eqn{ \sigma_a}, the prior standard deviation of \eqn{ E(Y)},
    indicating how much to smooth \eqn{ E(Y)} over age groups (which may
    vary over geographic areas and time periods, and with the standard
    deviations averaged over age groups). A larger standard deviation
    represents more prior uncertainty, which allows the data to play a
    greater role. (2) NA to not smooth in this way. (3) To have YourCast
    search for a good value based on a target value of the derivative of
    \eqn{ E(Y)} with respect to age, set to a vector of elements containing
    the start and end of a range in sigma in which to look (such as 0.05
    and 1.5), the number of values to look at within this range (such as
    5), and the target value of the derivative of \eqn{ E(Y)} with respect
    to age (such as 0.05). The vector may also include a fifth element,
    which is the target value of the total standard deviation of \eqn{E(Y)}
    over all dimensions of the prior (such as 0.1). (You may choose to
    run YourCast with model=EBAYES on a related data set to find an
    approximate target value of the derivative and standard deviation
    automatically.) Default: 0.30. }
  
  \item{Ha.sigma.sd}{A scalar; the standard deviation of parameter
    Ha.sigma (for Gibbs sampling only). Default: 0.1.}
  
  \item{Ha.deriv}{A numeric vector, each element of which is \eqn{n},the degree of a (discrete) derivative of the
    smoothness functional with respect to the age group. Element \eqn{k} of
    this vector refers to the \eqn{(k-1)}th derivative, where 0 excludes
    the derviative, 1 includes it, and values in between include the
    derivative but weight it down proportionally. The first element of
    the vector corresponds to the weight on the derivative with respect
    to age of order 0 (the identity operator), the second to the weight
    on the derivative of order 1 (the 1st derivative), etc. For example,
    c(0, 1, 1) corresponds to a mixed functional that penalizes the
    first and second derivatives equally. The higher the order of
    derivative, the more local smoothness over age groups; and lowest
    specified derivative controls the form of prior
    indifference. Default: c(0, 0, 1), which usually works well.}

  \item{Ha.age.weight}{A scalar or a numeric vector with weights that
    determine how much smoothing occurs for different age groups. If set
    to 0 or NA, age groups are weighted equally; if set to a nonzero
    scalar, the weight for age group \eqn{a} is set proportional to
    \eqn{a^Ha.age.weight};
    if a vector of length A, the \eqn{ a}th element is the
    weight of age group \eqn{ a}. Default: 0.}
  
  \item{Ha.time.weight}{A scalar or a numeric vector with weights that
    determine how much smoothing occurs for different time periods when
    smoothing over age groups. If 0 or NA, time periods are weighted
    equally; if set to a nonzero scalar value, the weight for time
    period \eqn{ t} in smoothing age groups is proportional to
    \eqn{t^Ha.time.weight}; if the argument is a vector of length T, the 
    \eqn{t}th element is the weight of time period \eqn{t}. Default: 0.}
  
  \item{Ht.sigma}{This can be set in one of three ways: (1) a scalar
    which sets \eqn{ \sigma_t}, the prior standard deviation of \eqn{E(Y)},
    indicating how much to smooth \eqn{E(Y)} over time periods (which may
    vary over geographic areas and age groups, and with the standard
    deviations averaged over time periods). A larger standard deviation
    represents more prior uncertainty, which allows the data to play a
    greater role. (2) NA to not smooth in this way. (3) To have YourCast
    search for a good value based on a target value of the derivative of
    \eqn{ E(Y)} with respect to time, set to a vector of elements containing
    the start and end of a range in sigma in which to look (such as 0.05
    and 1.5), the number of values to look at within this range (such as
    5), and the target value of the derivative of \eqn{ E(Y)} with respect
    to time (such as 0.05). The vector may also include a fifth element,
    which is the target value of the total standard deviation of \eqn{ E(Y)}
    over all dimensions of the prior (such as 0.1). (You may choose to
    run YourCast with model=EBAYES on a related data set to find an
    approximate target value of the derivative and standard deviation
    automatically.) Default: 0.30.}
  
  \item{Ht.sigma.sd}{A scalar; the standard deviation of parameter
    Ht.sigma (for Gibbs sampling only). Default: 0.1.}
  
  \item{Ht.deriv}{A numeric vector, each element of which is
    \eqn{n}, the degree of a (discrete) derivative of the
    smoothness functional with respect to time. Element \eqn{k} of this
    vector refers to the \eqn{(k-1)}th derivative, where 0 excludes the
    derviative, 1 includes it, and values in between include the
    derivative but weight it down proportionally. The first element of
    the vector corresponds to the weight on the derivative with respect
    to time of order 0 (the identity operator), the second to the weight
    on the derivative of order 1 (the 1st derivative), etc. For example,
    c(0, 1, 1) corresponds to a mixed functional that penalizes the
    first and second derivatives equally. The higher the order of
    derivative, the more local smoothness over time; and lowest
    specified derivative controls the form of prior
    indifference. Default: c(0, 0, 1), which usually works well.}

  \item{Ht.age.weight}{A scalar or a numeric vector with weights that
    determine how much smoothing occurs for different age groups when
    smoothing over time. If set to 0 or NA, age groups are weighted
    equally in smoothing over time; if set to a nonzero scalar, the
    weight for age group \eqn{a} is set proportional to \eqn{a^Ht.age.weight};
    if a vector of length A, the \eqn{ath} element is the weight of age
    group \eqn{a}. Default: 0.}
  
  \item{Ht.time.weight}{A scalar or a numeric vector with weights that
    determine how much smoothing occurs for different time periods when
    smoothing over time. If 0 or NA, time periods are weighted equally;
    if set to a nonzero scalar value, the weight for time period \eqn{t} in
    smoothing time periods is proportional to \eqn{t^Ht.time.weight}; if
    the argument is a vector of length T, the \eqn{t}th element is the
    weight of time period \eqn{t}. Default: 0.}
  
  \item{Hat.sigma}{This can be set in one of three ways: (1) a
    scalar which sets \eqn{ \sigma_{at}}, the prior standard deviation
    of \eqn{E(Y)}, indicating how much to smooth the time trend in \eqn{E(Y)} over
    age groups. A larger standard deviation represents more prior
    uncertainty, which allows the data to play a greater role. (2) NA to
    not smooth in this way. (3) To have YourCast search for a good value
    based on a target value of the derivative of \eqn{E(Y)} with respect to
    age and time, set to a vector of elements containing the start and
    end of a range in sigma in which to look (such as 0.05 and 1.5), the
    number of values to look at within this range (such as 5), and the
    target value of the derivative of \eqn{ E(Y)} with respect to age and
    time (such as 0.05). The vector may also include a fifth element,
    which is the target value of the total standard deviation of \eqn{E(Y)}
    over all dimensions of the prior (such as 0.1). (You may choose to
    run YourCast with model=EBAYES on a related data set to find an
    approximate target value of the derivative and standard deviation
    automatically.) Default: 0.2. }
  
  \item{Hat.sigma.sd}{A scalar; the standard deviation of parameter
    Hat.sigma (for Gibbs sampling only). Default: 0.1.}
  
  \item{Hat.a.deriv}{A numeric vector, each element of which is \eqn{n}, the degree of a (discrete) derivative of the
    smoothness functional of time trends with respect to age
    groups. Element \eqn{k}  of this vector refers to the \eqn{(k-1)}th
    derivative of the time trend v with respect to age, where 0 excludes
    the derviative, 1 includes it, and values in between include the
    derivative but weight it down proportionally. The first element of
    the vector corresponds to the weight on the derivative of the time
    trend with respect to age of order 0 (the identity operator), the
    second to the weight on the derivative of order 1 (the 1st
    derivative), etc. For example, c(0, 1, 1) corresponds to a mixed
    functional that penalizes the first and second derivatives
    equally. The higher the order of derivative, the more local
    smoothness over time; and lowest specified derivative controls the
    form of prior indifference. Default: c(0, 0, 1), which usually works
    well.}
  
  \item{Hat.t.deriv}{A numeric vector, each element of which is \eqn{n}, the degree of a (discrete) derivative of the
    smoothness functional of age derivative with respect to
    time. Element \eqn{k} of this vector refers to the \eqn{(k-1)}th
    derivative of the age derivative with respect to time, where 0
    excludes the derviative, 1 includes it, and values in between
    include the derivative but weight it down proportionally. The first
    element of the vector corresponds to the weight on the age
    derivative with respect to time of order 0 (the identity operator),
    the second to the weight on the derivative of order 1 (the 1st
    derivative), etc. For example, c(0, 1, 1) corresponds to a mixed
    functional that penalizes the first and second derivatives
    equally. The higher the order of derivative, the more local
    smoothness over time; and lowest specified derivative controls the
    form of prior indifference. Default: c(0, 0, 1), which usually works
    well.}
  
  \item{Hat.age.weight}{A scalar or a numeric vector with weights that
    determines how much smoothing occurs for different age groups when
    smoothing over age and time. If set to 0 or NA, age groups are
    weighted equally in smoothing over time; if set to a nonzero scalar,
    the weight for age group \eqn{a} is set proportional to
    \eqn{a^Ht.age.weight}; if a vector of length A, the \eqn{a}th element is the
    weight of age group \eqn{a}. Default: 0.}
  
  \item{Hat.time.weight}{A scalar or a numeric vector with weights that
    determine how much smoothing occurs for different time periods when
    smoothing over age and time. If 0 or NA, time periods are weighted
    equally; if set to a nonzero scalar value, the weight for time
    period \eqn{t} in smoothing time periods is proportional to 
    \eqn{t^Ht.time.weight}; if the argument is a vector of length T, the 
    \eqn{t}th element is the weight of time period \eqn{t}. Default: 0.}
  
  \item{Hct.sigma}{A scalar which sets \eqn{\sigma_t}, the prior standard
    deviation of \eqn{E(Y)}, which indicates how to smooth  \eqn{E(Y)} over
    geographic areas, or NA to not smooth in this way. The parameter 
    \eqn{\sigma_ct} is the expected prior standard deviation of \eqn{E(Y)} for a
    geographic area (varying over time periods and age groups, and with
    the standard deviations averaged over geographic areas). (A larger
    standard deviation represents more prior uncertainty, which allows
    the data to play a greater role.) Default: 0.3. }
  
  \item{Hct.sigma.sd}{A scalar; the standard deviation of parameter
    Ht.sigma (for Gibbs sampling only). Default: 0.1.}
  
  \item{Hct.t.deriv}{A numeric vector; controls whether smoothing the
    level or the time trend of \eqn{E(Y)} over geographic areas (both
    cannot presently be done simultaneously). To smooth the level of 
    \eqn{E(Y)} over geographic areas, set to 1, the identity. To smooth the
    time trend, set this (as in Hat.t.deriv) to the weight of the
    partial derivative taken with respect to time in the standard
    smoothness functional for the prior. The use of the first or higher
    order partial derivatives are supported. Default is 1.}
  
  \item{Hct.time.weight}{A scalar or a numeric vector with weights
    that determine how much smoothing occurs for different time periods
    when smoothing over geographic areas. If 0 or NA, time periods are
    weighted equally; if set to a nonzero scalar value, the weight for
    time period \eqn{t} in smoothing over areas is proportional to 
    \eqn{t^Hct.time.weight}; if the argument is a vector of length T, the 
    \eqn{t}th element is the weight of time period \eqn{t}. Default: 0. }
  
  
  \item{LI.sigma.mean}{A scalar; used in the likelihood and in the
    calculation of the priors in conjunction with Ha.sigma.sd,
    Hat.sigma.sd, Ht.sigma.sd, and Hct.sigma.sd. Default is 0.2.}
  
  \item{LI.sigma.sd}{A scalar; the standard deviation of LI.sigma.mean used in the calculation of the priors. Default is 0.1.}
  
  \item{nsample}{A scalar; represents the number of iterations in the
    Gibbs algorithm ``BAYES''. Default: 500.}
  
  \item{low.pow}{Boolean. Whether to include lower-power of explanatory
    variables in the simulation as derived from \code{formula}. For
    example \eqn{ y \sim x^4}, if \code{low.pow} = TRUE, then
    \eqn{x, x^2, x^3, x^4} will be included. Default TRUE.}

  \item{verbose}{Boolean.  Suppress verbose output.  Defaiult FALSE}
}
\details{
  The file dataobj.Rdata in the data folder is an example of the dataobj
  argument of yourcast.  
}
\value{
  Returns a list containing the following components: 

  \item{call}{The full call, including all command line options when
    yourcast was called.}
  
  \item{userfile}{The full userfile if it was specified}
  
  \item{yhat}{A list with the same cross-sectional elements as the input
    data, but with two columns, y for the observed dependent variable and yhat for the predicted values. These include both
    in-sample and out-of-sample values,as distinguished by the values of
    sample.frame.}
  
  \item{coeff}{A list with the same cross-sectional elements as the
    input data, elements of which are the estimated coefficients if
    calculated by the chosen model.}
  
    \item{sigma}{A list with the same cross-sectional elements as the
    input data, elements of which are the estimated standard error of
    the estimate of the regression (the standard deviation of the
    dependent variable given the explanatory variables).}
  
}
\references{\url{http://gking.harvard.edu/yourcast} }

\author{ Federico Girosi <\email{girosi@rand.org}>; Elena
  Villalon<\email{evillalon@iq.harvard.edu}>; 
Gary King<\email{king@harvard.edu}}
\note{ 
See the documentation on line: \url{http://gking.harvard.edu/yourcast/docs}
 ~Make other sections like Warning with \section{Warning }{....} ~
}
\seealso{ ~~objects to See Also as \code{\link{help}}, ~~~ }
\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.

## The function is currently defined as
function(formula=NULL, dataobj=NULL,
                     sample.frame=c(time=1950, 2000, 2001, 2030), 
                     standardize=TRUE, elim.collinear=FALSE,
                     tol=0.9999, solve.tol = 1.e-10,svdtol=10^(-10),
                     userfile=NULL, savetmp = T, model.frame=F,
                     debug = F,  rerun= "yourcast.savetmp", 
### specific to models
                     model="OLS",zero.mean=FALSE,
###                     ols.sigma.param=list(sigma.bar=1,use.deaths=FALSE,average=TRUE,model.based=FALSE), 
          #### smooth over ages           
                     Ha.sigma = 0.3,
                     Ha.sigma.sd= 0.1, Ha.deriv=c(0,0,1),
                     Ha.age.weight=0, Ha.time.weight=0,
          #### smooth over time
                     Ht.sigma= 0.3,
                     Ht.sigma.sd=0.1,  Ht.deriv=c(0,0,1),
                     Ht.age.weight=0, Ht.time.weight=0,
          #### smooth over age-time
                     Hat.sigma=0.2,
                     Hat.sigma.sd=0.1,Hat.a.deriv=c(0,1),Hat.t.deriv=c(0,1), 
                     Hat.age.weight=0,Hat.time.weight=0,
          #### smooth over cntry-time
                     Hct.sigma=0.3, Hct.sigma.sd =0.1,
                     Hct.t.deriv=1, Hct.time.weight = 0,
                     LI.sigma.mean=0.2,LI.sigma.sd = 0.1, nsample= 500, low.pow=T, verbose=TRUE)
{
  #USER INPUT ERROR CHECKING
 #Load lpSolve package and print error if it is not available
  if (!require(sma, quietly = TRUE))  {
    stop("The sma package is required.\nInstall it from CRAN.")
  }
  guirun <- NULL ###provision to run the simulation with a GUI
### done with data input check model  
   
### some initialization and clean up
  try(rm(env.base, env.who, inherits=T), silent=T);
### I'm defining base environment
  env.base <- environment();
  
  if(is.character(model))
    model <- trim.blanks(toupper(model))
  if(length(dataobj) > 0 )
    dataobj0 <- dataobj
  if(is.character(dataobj))
    dataobj0 <- trim.blanks(dataobj0) ### saving the initial input

  rerun <- trim.blanks(rerun)
  if(length(rerun) > 0 && !is.character(rerun))
    stop(rerun)
  
  if(length(userfile) > 0)
    messout(paste("Inputs in userfile ", userfile, "overwrite yourcast arguments"), verbose)
          
### previous run of yourcast may be stored in file
###  print(formals(yourcast)$Ha.sigma)
###  print(match.call(yourcast))
###  print(str(match.call(yourcast)))
 
  if((length(dataobj$data) <= 0 || length(formula) <= 0) && 
     !file.exists(rerun) && length(userfile) <= 0)
    stop("Insufficient data")

   
### for debug = T then make it available as global
  if(debug){
    messout("Setting to mode debugging with the global env.base", verbose)
    env.base <<- env.base
  }
  
##parsing for insample and outsample periods year
  yrest <- NULL
  if(length(sample.frame) > 0)
    yrest <- find.yrest(sample.frame)
    
  year.digits <- nchar(yrest)

### get the number of digist for each of cntry,yourcast.savetmpages and years
### from the time cross sectional identifiers are stored in
### dataobj$index.code, parse it and assign them to env.base
  
  if(length(dataobj$index.code) > 0){
    ix.code <- parse.index.code(icode=dataobj$index.code, yrest)
    assign.number.digits(ix.code, env.base)    
  }

  Hct.c.deriv <-  NULL
  if(length(dataobj$proximity) >0)
    Hct.c.deriv <- dataobj$proximity
  
### check if the user has provided other inputs with reflection
### give the name of the driver and call for the simulation
  
  driver <- match.call()

### print(driver)
  driver <- as.character(driver)
### name of the calling function 
  mortality.driver <- driver[1]
### print(mortality.driver)
### give me its arguments
 
  args  <- names(formals(mortality.driver))
  names(args) <- args
  
###  print(args) 
###
### is the call coming from the global environment or from a GUI
### if from command line; creates env.who and stored all input data there
  
  if(length(guirun) <= 0  || !is.environment(guirun)){
    env.who <- WHO.yourcast(args, env.base)
    sims <- get("N", env=env.who)
    
  }else 
  env.who <- guirun

### find out the arguments of yourcast to be added to output

  callst <- call.yourcast(formlst=formals(mortality.driver),
                          callmatch= match.call(), model.frame, rerun)

### exclude dataobj (way too big) if model.frame =F,i.e. include just its name
   callst <- call.dataobj1(callst, model.frame, env.who)
 
### exclude dataobj (way too big)  

### tobe added to output
  userfilelst <- NA
  if(length(userfile) > 0){
    userfilelst <- args.userfile(userfile,env.base)
    messout("Retreiving userfile parameters for...",verbose)
    messout(paste(names(userfilelst),sep=" "),verbose)
    
     
  }
### finds out the inputs for dataobj, if any.  Process them
### It migth be either a file or an object in memory; assign to env.base

  if(length(dataobj) > 0 ) {

    dataobj <- parse.dataobj(dataobj,env.base)
        
  }
###last check the userfile and if it is not null, then its inputs
### parameters will overwrite the arguments of yourcast
  if(any(!is.na(userfilelst)) )
    args1 <- try(get("userfile", env.who), silent=T)
  else
    args1 <- NULL

### update args with the user supplied file, if any
  if (class(args1) != "try-error" && !is.null(args1)){
    messout("Updating global with the userfile parameters", verbose)
    
    update.args(args, userfile, env.base);
    
    env.who <- WHO.yourcast(args, env.base, env.who)
    
  }
 
### rownames instead of tagging with years change to
### time cross-sectional id's for consistency with old code
 if(length(dataobj$data) > 0)
   {
     dataobj <- sort.dataobj(dataobj)
     ix <- 1:length(dataobj$data)
     names(ix) <- names(dataobj$data)
     data <- lapply(ix, FUN="elimallna.csid",dataobj$data, verbose)
     data <- conversion.rownames(dataobj$data, year.digits)
     dataobj$data <- data  
   }
###sanity checks
  if(is.character(userfile) && !file.exists(userfile))
    if((length(unlist(dataobj)) <= 0 || length(formula) <=0)
       && !file.exists(rerun))
    stop("Insufficient data")
 
### if no dataobject and formula use previous run
### which might be stored in the file yourcast.savetmp
  objread <- FALSE
  if((length(unlist(dataobj)) <= 0 || length(formula) <=0) && !is.character(userfile))
     {
     dataobj <- parse.dataobj(rerun,env.base)
     objread <- TRUE
   }
 
### done with data input check model  
  if(is.character(model))
    model <- trim.blanks(toupper(model))  
 ### for consistency with past code; taking the log with model poisson
  if(length(formula) > 0)
    {
      log.poiss <- check.depvar(formula)    
      assign("log.poiss", log.poiss, env=env.who)
    
    }
   
### parameters args are stored correctly in env.who;
### bring them locally for easy of computation.
### This will only run if we have dataobj$data and formula
###################################################################
### RUN THE PREPROCESSING, which output are the lists variables
### Estimate of whoinsampy, whoutsampy, whocov, whoinsampx, whoutsampx
  prepross <- NULL ##preprocessing 

  if(length(dataobj) > 0 && !is.character(dataobj)) ##preprocessing
    {
    
      if(length(ind <-grep("lag", formula))> 0 )
        {
          lst <-  lagdataobj(formula, dataobj)
          datamat <- lst$datamat
          formula <- lst$ff
          dataobj$data <- datamat
 ###       print(formula)
        }
 
      prepross <- input.to.model(datamat=dataobj$data,ff=formula,
                                 all.pow=low.pow, sample.frame,
                                 index.code=dataobj$index.code,
                                 Gnames= dataobj$G.names,  
                                 standard=standardize, elim.collinear,
                                 tol=tol, solve.tol=solve.tol,
                                 log.poiss=log.poiss,verb=verbose)
 
      if(class(Hct.c.deriv)!= "try-error" && length(Hct.c.deriv) > 0)
        hc <- Hct.c.deriv
      else 
        hc <- dataobj$proximity
      
      assign.to.env(prepross, model, prox = hc, ewho=env.who);
### if savetmp =T then saves the prepross list return by input.to.model
### into a file in the working directory whose name is given by parameter rerun
      
      if(savetmp)
        save.yourcast.file(rerun, env.base)
            
      filexists <- try(file.exists(dataobj0),silent=T)
      if(class(filexists) != "try-error")
        if(filexists && !identical(dataobj0, rerun))
          dataobj <- parse.dataobj(rerun, env.base)
         
    }### done with preprocessing
##########################################################
    
### RUN MODEL REGRESSION ANALYSIS 
 
     
### sanity checks 
  if(model=="LS") model <- "OLS";
  if(length(formula) > 0) 
    depvar <- leftside.formula(formula)$numerator
  
  if((model=="MAP" || model=="EBAYES") && !is.na(Hct.sigma) ){
    messout("Setting Hct.sigma = NA...",verbose)
    Hct.sigma <- NA
    assign("who.Hct.sigma", Hct.sigma, env=env.who)
  }
    
    
  if((model=="BAYES" || model=="MAP" || model=="EBAYES") && !is.na(Hct.sigma))
    who.Hct.c.deriv <- proximity.fill(dataobj,get("who.Hct.c.deriv", env=env.who))
    
   
  if(model=="MAP" || model=="EBAYES"){
 
    autoset <- c(Ha.sigma["d1.a"], Ht.sigma["d1.t"], Hat.sigma["dtda"])
 
    autoset <- find.all.args()
    autoset <- unlist(autoset)
    assign("autoset", autoset, env=env.who)
    summary.measures <- NULL
    if(length(autoset) > 0){
      summary.measures <- names(autoset)
      assign("summary.measures", summary.measures, env=env.who)
      messout("Printing autoset....",verbose, obj=unlist(autoset))
     
    }
    
  }
   
  ### which model are you running ??
 
  indbayes <- grep(model, c("MAP", "BAYES", "EBAYES"))
### consistency check for zero.mean
### and reads a file with vector value for zero.mean
  if(length(indbayes) > 0)
    zero.mean <- zero.mean.age.profile(indbayes, env.who)
  
  smooth1 <- get.param.smooth(env.who)### c(Ha.sigma, Ht.sigma, Hat.sigma)
  if(length(Ha.sigma) >= 3 || length(Ht.sigma) >= 3 || length(Hat.sigma) >= 3)
    smooth <-  NULL
  else
    smooth <- smooth1
 
 
  dfirst  <- get("who.digit.first", env=env.who)
  cdigits <- get("who.cntry.digits", env=env.who)
  whoinsampy  <- get("whoinsampy", env=env.who)
 
  count.cntry <- unique.default(unlist(sapply(names(whoinsampy), substr, dfirst, cdigits)))
  G <- rbind(NULL, smooth1)

  final.sigmas <- NULL
  if(length(indbayes) > 0 && length(smooth) < 3){
      G <- build.prior.param(smooth,count.cntry, Ha.sigma.vec, Ht.sigma.vec, Hat.sigma.vec, sims,
                             stats=c(d1.a=d1.a.stat,d1.t=d1.t.stat,dtda=dt.da.stat),
                             autoset, model, env.who)
      final.sigmas <- G[1, ]
      messout("\n",verbose, obj=final.sigmas)
      
    }
  modeltopass <- model
   
  if(identical(model,"EBAYES"))
    modeltopass <- "MAP"
  
  ix <- grep(model, c("OLS", "LS", "POISSON", "LC", "BAYES", "EBAYES"))
  bool <- (length(Ha.sigma) >= 1 || length(Ht.sigma) >= 1 || length(Hat.sigma) >= 1)
 

  if(length(count.cntry) >  1 || length(ix) > 0 || bool){
   
    m <- try(switch(modeltopass, OLS=ols(env.base),LC=lc(env.base),POISSON=glm.poisson(env.base),
                       MAP=cxc(ebase=env.base),
                       BAYES=gibbs.sampler(), model="", NULL), silen=F);
    params <- NULL
  }else{
    ### runs one for one cntry and model MAP
   
    lst <- onecntry.bayes.empirical(G)
    m <- lst$m
    params <- lst$params
  
  }

  if(class(m) =="try-error")
    messout("Data is inconsistent.",verbose)

  if (is.null(m))
    stop(paste("Model ",model," is not available"));
       
### m contains obj= yhatin, yhatout, insampy, outsampy but listed with csid
### conversion.cntry.mat uses list.by.cntry to convert into country list matrices
### so it is a more useful format for the graphics output.
  
### EBAYES specific; calling Federico code
  if(identical(model, "EBAYES")){
 
    formula <- get("formula", env=env.who)
   
    mebayes <- model.ebayes(autoset,formula,m,summary.measures,
                            depvar,graphics.file=NA,env.who)
   
    names(mebayes$summary) <- NULL ### to avoid problems with outputs
    messout("\n",verbose,obj=unlist(mebayes$summary))
    return(mebayes)
  }
  
  model <- model.string()
  
  outy <- yhat.mat(ebase=env.base)
  coeff <- m$coeff
  std <- m$std
  
  outputlist <- list(yhat=outy,call=callst,userfile=userfilelst,
                     coeff=coeff, sigma=std)
   if(length(params) <= 0)
     params <- c(Ha.sigma=Ha.sigma, Ht.sigma=Ht.sigma, Hat.sigma=Hat.sigma)
  
   outputlist <- c(outputlist, list(params=params))
   
  if(get("save.output", env=env.who)  && !is.na(model)){
    messout("Saving output file (option save.output = TRUE", verbose)
    depvar <- leftside.formula(formula)
    build.file.output(outputlist,model, depvar);
  }
     
### some formatting for yhat
  
  outy <- format.yhat(outy,year.digits)
  outputlist$yhat  <- outy
  if(length(final.sigmas) > 0)
    outputlist$final.sigmas <- final.sigmas
     
  return(invisible(outputlist)) 
}

}
\keyword{file}% at least one, from doc/KEYWORDS
\keyword{datasets}% __ONLY ONE__ keyword per line
