\name{yourgraph}
\alias{yourgraph}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Summary graphs for yourcast}
\description{
  Generates summary graphs for the output object of \code{yourcast}. The
  two possible type of graphs are the time series of the data and forecasts for each
  age group and every cross-section. Age profiles observed and predicted for each year and cross-section.
}
\usage{
yourgraph(input.data, obs.insample= T, obs.outsample = T,
          pred.insample = T, pred.outsample = T, wide = 8.5,high = 8.5, plot.time.series = T,
          plot.age.profiles = T, page.horizontal = FALSE, row.num = 1, col.num = 2, background.col = "tan1",
          print2file = T, dv.transform = function(x){x}, depvartext = long.causes, strata=NULL,
          dirpath=NULL, ylim=NA,dispgraph=options()[["pdfviewer"]], verbose=T, demoFile=NULL)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{input.data}{A list. The output from \code{yourcast}, which could
    either be in a file or an object.}
  \item{obs.insample}{A list.Observed data for the in-sample period. Default:TRUE }
  \item{obs.outsample}{A list. Observed data for the out-sample period. Default:TRUE}
  \item{pred.insample}{A list.Predicted data for the in-sample period. Default:TRUE}
  \item{pred.outsample}{A list.Predicted data for the out-sample
    period. Default:TRUE }
  \item{wide}{Scalar.Width of the graphics region in the postscript file.Default: 8.}
  \item{high}{Scalar.height of the graphics region in the postscript
    file. Default: 8. }
  \item{plot.time.series}{Boolean. Whether or not to plot the time series graph. Default: TRUE }
  \item{plot.age.profiles}{Booleean. Whether or not to plot the age
    profiles graph. Default: TRUE}
  \item{page.horizontal}{ Boolean. Whether orientation of the
    image in the postscript file. Default:FALSE.}
  \item{row.num}{Scalar. Number of rows in the graphs. Default: 1. }
  \item{col.num}{Scalar. Number of columns in the graphs. Default: 2. }
  \item{background.col}{String. Background color for the postscript file.Default: "tan1".}
  \item{print2file}{Boolean. Whether output should be printed to a pdf
    and  eps files (TRUE) or to the screen (FALSE). Default:TRUE) }
  \item{dv.transform}{Transformation to be performed on the data. Default: \eqn{function(x){x}}.}
  \item{depvartext}{Name of a function that contains labels for the
    graphics. Default: long.causes}
  \item{strata}{String. "m" for male and "f" or no value NULL. Default: NULL}
  \item{dirpath}{String. Directory path to be appended to 
    \code{input.data} to find the file object input for the simulation.Default:NULL }
  \item{ylim}{Scalar. Argument to \R function matplot. Default: NA}
  \item{dispgraph}{String. The OS specific device to display
    graphics. Default:\eqn{options()[["pdfviewer"]]}.}
  \item{verbose}{Boolean.Whether to display verbose output. Default: TRUE}
  \item{demoFile}{String. Namer of file to create the demo's. Default: NULL}
}
\details{
  ~~ If necessary, more details than the description above ~~
}
\value{
  Handle to the graphics created and displayed in the screen.
 
}
\references{ \url{http://gking.harvard.edu/yourcast}  
\author{Federico Girosi <\email{girosi@rand.org}>; Nirmala Ravishankar <\email{nirmalar@u.washington.edu}} }
\note{ ~~further notes~~ 
 See demos; for example: demo(chp.11.4)
 ~Make other sections like Warning with \section{Warning }{....} ~
}
\seealso{ ~~objects to See Also as \code{\link{help}}, ~~~ }
\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.

## The function is currently defined as
function(input.data, obs.insample= T, obs.outsample = T,
                      pred.insample = T, pred.outsample = T, ### 8
                      wide = 8.5,high = 8.5, plot.time.series = T,
                      plot.age.profiles = T, page.horizontal = FALSE,
                      row.num = 1, col.num = 2, background.col = "tan1",
                      print2file = T, dv.transform = function(x){x},
                      depvartext = long.causes, strata=NULL,
                      dirpath=NULL, ylim=NA,dispgraph= options()$pdfviewer,
                      verbose=T, demoFile=NULL)
 

  {
###    book: If true it generates one '.eps' file for each country and each
###          time series and age profiles. You need to set either
###          plot.time.series = F or  plot.age.profiles = F, to generate
###          separate figures for each plot.  Default false.
###            
  book <- FALSE
    ##Creating directory path name

    ev <- environment()
    if(length(dirpath) <= 0)
      dirpath <- paste(getwd(), "/",sep="")
    else
      dirpath <- paste(dirpath,"/", sep="")
    ### for files titles
    if(length(demoFile) > 0) demoFile <- paste(demoFile,"_", sep="")
    ## Next we load the input file, which is usually the output from your.cast().
    ## It can either be a data file or an object. If it is not generated
    ## from your.cast(), then it should be configured as per the specifications
    ## of the yourcast object.
  
    if (is.character(input.data)){
      messout("Reading in data file...", verbose)   
      file <- paste(dirpath, input.data, sep="")
      input.data <- load(file, envir=ev)
      if(is.symbol(input.data))
        input.data <- eval(as.symbol(input.data))
    } else 
    messout("Obtaining data object...", verbose)
  
    data.vec <-  c("cntry.lst", "depvar", "strata", "insampy", "outsampy",
                 "yhatout", "yhatin", "model")
   
    input.data <- convert.input.graph(input.data, strata)
  
    cntry.lst  <- input.data$cntry.lst
 
    depvar <- input.data$depvar
    model  <- input.data$model
    strata <- input.data$strata
    insampy  <- input.data$insampy
    outsampy <- input.data$outsampy
    yhatin   <- input.data$yhatin
    yhatout  <- input.data$yhatout
  
    data.sym <- sapply(data.vec,as.symbol)
    check <- sapply(data.sym, eval, env=ev)
  
    insampy  <- lapply(insampy,FUN = dv.transform) 
    outsampy <- lapply(outsampy,FUN = dv.transform)
    yhatin   <- lapply(yhatin,FUN = dv.transform)
    yhatout  <- lapply(lapply(yhatout, na.omit), FUN = dv.transform)
 
 ## Creating disease, gender and country name strings for labels
    disease.name <- NULL
    depvarN <- extract.depvar.graph(depvar)$numerator

    if(is.function(depvartext)){
      lname <- depvartext(depvar)
 
      if(!identical(lname,depvar) ){
        lname <- depvartext(depvarN)
        disease.name <- ifelse( is.list(lname), lname[[1]], lname[1])
   
      }
    }
    if(length(disease.name) <= 0)
      disease.name <- depvar
  
    nm <- names(cntry.lst)
    ln <- nrow(cntry.lst)
  
  
    if(length(ln) > 0)
      {
        countries <- as.matrix(cntry.lst[,1])
        c.name <- as.matrix(cntry.lst[,2])
      }else{
        ln <- 1
        countries <- cntry.lst[1]
        c.name <- cntry.lst[2]
      }

    gender.str0 <- NULL
    gender.str <- "_"
    if (length(strata) > 0 && is.numeric(strata)){
      if (strata==2) {
        gender.str <- "_m_"
        gender.str0 <- "m"
      }
      else if (strata==3){
        gender.str0 <- "f"
        gender.str <- "_f_"
      } else if (length(strata) > 0){
        gender.str0 <- strata
        gender.str <- paste("_",strata,"_",sep="")
      }
    }

    ##The following does the actual plotting using plot.cast.new()
    if (sum(pred.insample == T, pred.outsample == T,
            obs.insample == T, obs.outsample ==T) == 0){
    ## The above checks if there is any data to plot.
      messout("No data found for plotting.", verbose)

      return(list())
    }
    if (print2file == F || !length(print2file)){
      messout("Set option print2file to true or a name", verbose)

      return(list())
    }
    
    comm <- paste(depvarN, gender.str, model,sep="")
    
    if(length(demoFile) > 0)
      comm <- paste(demoFile,comm,sep="")
    
    if(!is.character(print2file)){
      
      outfilename <- paste(comm,".pdf", sep = "")
      outfilepost <- paste(comm,".eps", sep = "")
      
    }else{
      outfilename <- paste(print2file,".pdf", sep = "")
      outfilepost <- paste(print2file,".eps", sep = "")
    }
    
    outfile <- paste(dirpath, outfilename, sep = "")
    outpost <- paste(dirpath, outfilepost, sep = "")
    ## Creating the output file
    
    messout(paste("The file ", outfilename," written to disk...",sep=""),verbose)
    if(!book)
      messout(paste("The file ",outfilepost," written to disk...",sep=""),verbose)
    
###first, poscript files always generated
    if(file.exists(outpost))
      file.remove(outpost)
    
    high <- high * row.num
    wide <- wide * col.num
    if(!book){ ### create the postscript file  
      file.create(outpost)
   ### federico has width=heigth=high
   ###   postscript(outpost,paper="special",pointsize = 14,width=height,
   ###              height=height,horizontal= page.horizontal,bg= background.col)
   ### good,pointsize = 13,width=8.5,height=11,
   ### for postscript ,horizontal = T
     
      postscript(outpost,paper="special",pointsize = 14,width=wide,
                 height=high,horizontal= page.horizontal,bg= background.col)
      par(mfrow = c(row.num,col.num))
      par(cex.main = 1.3, cex.lab = 1.2, cex.axis = 1.4,font=2,font.axis=2,font.lab=2)
      par(mar = c(5, 4, 2, 0)+1*c(1,0,1,1))
      dpos <- dev.cur()
    }
#### needed to generate ech graph for the book
    outposteach <-1:ln
    run <- 1
    
    plot.time.series0  <- ifelse(book, T, plot.time.series)
    plot.age.profiles0 <- ifelse(book, T, plot.age.profiles)
    if(book && plot.time.series && plot.age.profiles){
      message("For single files for each plot set one of plot.time.series=true or plot.age.profiles=true")
    }
    if(book && plot.time.series){
      
      for (i in 1:ln){
        comm <- paste("ts_",depvarN,gender.str, model,sep="")
        if(length(demoFile) > 0) comm <- paste(demoFile, comm,sep="")
        outfileposteach <- paste(comm,"_",c.name[i],".eps", sep = "")
        outposteach[i]  <- paste(dirpath, outfileposteach, sep = "")
        messout(paste("The file ",outposteach[i]," written to disk...",sep=""),verbose)
        file.create(outposteach[i])
        }
      
      }
    if(book && plot.age.profiles){
      
        for (i in 1:ln){
          comm <- paste("ap_",depvarN,gender.str,model,sep="")
          if(length(demoFile) > 0) comm <- paste(demoFile, comm,sep="")
          outfileposteach <- paste(comm,"_",c.name[i],".eps", sep = "")
          outposteach[i] <- paste(dirpath, outfileposteach, sep = "")
          messout(paste("The file ",outposteach[i]," written to disk...",sep=""),verbose)
          file.create(outposteach[i])
        }
        
      }
  ###  print(dev.list())
      for (i in 1:ln){
        
        if(book){
          postscript(outposteach[i],paper="special",pointsize = 14,width=wide,
                     height=wide,horizontal= page.horizontal,bg= background.col)
          par(mfrow = c(1,1))
          par(cex.main = 1.3, cex.lab = 1.2, cex.axis = 1.4,font=2,font.axis=2,font.lab=2)
         
           par(mar = c(4, 5, 2, 1)+1*c(1,0,1,0))
    
          dpos <- dev.cur()
        }
        pred.in  <- yhatin[[i]]
        pred.out <- yhatout[[i]]
        obs.in  <- insampy[[i]]
        obs.out <- outsampy[[i]]
       
      ##
        cntry.str <- c.name[i]
    
      ## Setting graphing parameters like background color, layout of the page 
     
###    par(bg= background.col);
   
        try(plotcast.new(pred.in, pred.out, obs.in, obs.out, disease.name, gender.str0,
                         cntry.str, print2file,plot.time.series,pred.outsample,
                         pred.insample,obs.outsample,obs.insample, plot.age.profiles,
                         ylim=ylim, verb=verbose), silent=T)
        
      }
  
   
    dev.off() ###closes the poscript device
    
    
    if (file.exists(outfile))
      file.remove(outfile)
      
    file.create(outfile)
     
    pdf(outfile,pointsize = 14,width=wide,family="Helvetica",
        bg =background.col, height=high)
      
    par(mfrow = c(row.num,col.num))
    par(cex.main = 1, cex.lab = 0.75, cex.axis = 0.5)
    par(mar = c(4, 4, 2, 1))
  
      ## The following code extracts the data for each country, formats it and then
      ## uses plot.new.graph() to plot the selected graphs.
    for (i in 1:ln){
        
      pred.in <- yhatin[[i]]
      pred.out <- yhatout[[i]]
      obs.in <- insampy[[i]]
      obs.out <- outsampy[[i]]
      
        ##
      cntry.str <- c.name[i]
      
        ## Setting graphing parameters like background color, layout of the page 
     
###    par(bg= background.col);
        
      try(plotcast.new(pred.in, pred.out, obs.in, obs.out, disease.name, gender.str0,
                       cntry.str, print2file,plot.time.series0,pred.outsample,
                       pred.insample,obs.outsample,obs.insample, plot.age.profiles0,
                       ylim=ylim, verb=verbose), silent=T)
      
    }
   
   
      dpdf <- dev.cur()
    
      dev.off()  
    
    
    op <- .Platform$OS.type
    if(identical(op,"unix") || identical(op,"linux")){
      dispgraph <- options()$pdfviewer 
      graph.dev <- paste(dispgraph, outfile, "&")
      res <- try(system(graph.dev,intern=F))
    }else if(identical(op, "windows"))
      res <- try(shell.exec(outfile))
  
###  res <- try(readLines(fifo(paste(dispgraph, outfile))), silent=T)
}

}
\keyword{file}% at least one, from doc/KEYWORDS
\keyword{graphs}% __ONLY ONE__ keyword per line
