\name{yourprep}
\alias{yourprep}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Data object for demos}
\description{
  Builds the data object from files names in some directory}
 
\usage{
yourprep(dpath= NULL, dataobj=NULL, icode = "GGGGAATTTT",
         str= c(index.code ="index.code",A.names="A.names",G.names="G.names",
         T.names="T.names",proximity="proximity",data="data"), verb=TRUE )
%- maybe also 'usage' for other objects documented here.
}
\arguments{
  \item{dpath}{String. Name of the directory that may have the data
    files. If the default then is the current directory. Default:NULL}
  
  \item{dataobj}{dataobj input to yourcast, it may contain some data
                 that will take preference from any of the files in
			  dpath. Default NULL}
   				 
  \item{icode}{String as in yourcast for the csid time series identifiers
    combination of country code("g"), age group("a"), and years("t").
    Default="ggggaattt"}
  
  
  \item{str}{ Vector of Strings with the name of the elements in list dataobj.}
   
  
  \item{verb}{Boolean for verbose output. Default:TRUE.}
}
 
  \details{
    Builds dataobj input to yourcast from directory files. The files
    names must be the csid identifiers such as combination of country
    country code and age group, for instance for US and age 45, file
    name should be "245045". The files must contained the full
    time-series values as especified in the variable sample.frame in
    yourcast. Say, if \eqn{sample.frame=c(1950,2000,2001,2030)}
    Then files have years of observations that may starts between 1950
    and 2000, and after the first observed year all other years must be
    included, even if the entries are NA, up to the last year of
    prediction, e,.g. 2030.  We have in the folder data three examples
    (245045, 418050, 428025) of files that yourprep will read folder to
    build the dataobj input to yourcast. 
  
}
\value{
  A list.
  \item{dataobj}{A list, see \code{yourcast} for details.
    The list contains several components. The \code{data} component is a
    list, which every element is a matrix for each cross-sectional unit 
    The csid identifier is a combination of country code and age group.  For
    example USA, country code 2450 and for age-group 45, is csid=245045. The
    columns of the matrices are time-series data for the dependent
    variables (i.e. disease, pop) and the covariates that will be used in 
    the simulation. Other components are \code{index.code} or
    \code{icode}, and \code{G.names} or \code{Gnames}.}
  
  \item{proximity}{ An object with three columns, the first and second
    are country codes and the third is the proximity value,i.e. 0, 1,
    2. See \code{proxfile}.}
  
  ...
}
\references{ \url{http://gking.harvard.edu/yourcast}  }
\author{ Elena Villalon<\email{evillalon@iq.harvard.edu}>}
\note{ ~~further notes~~ 
 See demos; for example: demo(chp.11.4)
 ~Make other sections like Warning with \section{Warning }{....} ~
}
\seealso{ ~~objects to See Also as \code{\link{help}}, ~~~ }
\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.
## The function is currently defined as

function(dpath= NULL, dataobj=NULL, icode = "GGGGAATTTT",
                        str= c(index.code ="index.code",A.names="A.names",G.names="G.names",
                          T.names="T.names",proximity="proximity",data="data"), 
                       verb=TRUE )
  {
    
### dataobj should be a list of elements whose names must be in the str vector     
   
    nmdobj <- NULL
    verbose <- verb 
    if(length(dpath) > 0 &&  length(grep("/$",dpath) ) <= 0)
      dpath <- paste(dpath,"/", sep="")
    
    if (length(dpath) <= 0)
      {
###set current directory as default
        dpath <- getwd()
        if(length(grep("/$",dpath) ) <= 0)
          dpath <- paste(dpath, "/", sep="")
      }
### if some elements are provided as inputs, make sure are the correct names
### they also take preference over any other files in the dpath or datadir 
    if(length(dataobj) > 0)
      {
        if(!is.list(dataobj))
          stop("Bad input: dataobj is either null or a list")
        
        nmdobj <- names(dataobj)
        bool <- sapply(nmdobj, is.element, str)
        if(any(bool==F))
          stop("Bad input: names of list dataobj are not correct")        
        allnames <- sapply(str,is.element, nmdobj)
      }
    
    mod <- FALSE
    mod <- GAT.names(dpath, str["G.names"],nmdobj, verbose)
    if(mod){
      Gfile <- paste(dpath, str["G.names"], sep="")
       
      dataobj$G.names <- build.Gnames(Gfile,verb=verbose)
  
    }

    mod <- FALSE
    mod <- GAT.names(dpath, str["A.names"],nmdobj,verbose)
    if(mod){
      Afile <- paste(dpath, str["A.names"], sep="")
      dataobj$A.names <- build.Gnames(Afile,verb=verbose)
     }
    
    mod <- FALSE
    mod <- GAT.names(dpath, str["T.names"],nmdobj,verbose)      
    if(mod){
      Tfile <- paste(dpath, str["T.names"], sep="")
      dataobj$T.names <- build.Gnames(Tfile,verb=verbose)
    }
    
    mod <- FALSE
    mod <- GAT.names(dpath, str["proximity"],nmdobj,verbose)      
    if(mod){
      proxfile <- paste(dpath, str["proximity"], sep="")
###      print(proxfile)
      dataobj$proximity <- build.Gnames(proxfile, nocol=3,verb=verbose)
    }
    
    mod <- FALSE
    mod <- GAT.names(dpath, str["index.code"],nmdobj,verbose)      
    if(mod){
   
      icodefile <- paste(dpath, str["index.code"], sep="")
      dataobj$index.code <- build.Gnames(icodefile, nocol=1,verb=verbose)
    }else
      if(!is.element(str["index.code"], nmdobj))
        dataobj$index.code <- icode
    
    datadir   <-  paste(dpath,str["data"], sep="")
    datafiles <- NULL
    if(file.exists(datadir))
      datafiles <- list.files(path=datadir, full.names=TRUE, recursive=TRUE)
    else{
      messout("Directory data not provided...", verbose)
      messout(paste("Reading dataobj$data files from ", dpath, sep=""),verbose)
    }

    if(length(datafiles) <= 0){
      datadir <- dpath
      datafiles <- list.files(path=datadir, full.names=TRUE, recursive=TRUE)
      ix <- grep(str["data"], str)
      if(length(ix) > 0)
        strdir <- str[-ix]
      else
        strdir <- str
      
      if(length(grep("/$",datadir)) <= 0)
        strdir <- sapply(str,function(nm) paste("/", nm,sep=""))
       
      files.to.exclude <- sapply(strdir,function(nm) paste(datadir,nm,sep=""))
      datafiles <- setdiff(datafiles, files.to.exclude)

    }
   ix <- grep(str["data"], str)
   str0 <- str
   if(length(ix) > 0)
     str0 <- str[-ix]
    
   if(length(datafiles) > 0 && !is.element(str["data"], nmdobj))
      dataobj$data <- build.data(datafiles, icode=dataobj$index.code,goodfiles=str0,verb=verbose)
   else if(!is.element(str["data"], nmdobj))
     messout("No data files provided for the simulation...", verbose)
     

   return(dataobj)
      
  }
}

\keyword{file}% at least one, from doc/KEYWORDS
